//
//  NowPlayingSaverView.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 13.9.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "NowPlayingSaverView.h"

#import "ArtworkAnimator.h"
#import "ArtworkCircleAnimator.h"
#import "ArtworkDoorwayAnimator.h"
#import "ArtworkFadeOutInAnimator.h"
#import "ArtworkFlipAnimator.h"
#import "ArtworkFromSideAnimator.h"
#import "ArtworkDrillAnimator.h"
#import "ArtworkScaleDownUpAnimator.h"
#import "ElementProvider.h"
#import "iTunes.h"

#import <dispatch/dispatch.h>



@implementation NowPlayingInfo

-(NSString*)description
{
    return [NSString stringWithFormat:@"<%@ trackName:%@ trackNumber:%d albumName:%@ artistName:%@ albumArtistName:%@ dbId:%d>", self.class, self.trackName, self.trackNumber, self.albumName, self.artistName, self.albumArtistName, self.dbId];
}

@synthesize trackName;
@synthesize trackNumber;
@synthesize albumName;
@synthesize artistName;
@synthesize albumArtistName;
@synthesize dbId;
@synthesize artwork;
@synthesize duration;
@synthesize position;

@end



#pragma mark -


@interface NowPlayingSaverView ()

-(NSString*)formatTrackInfo:(NowPlayingInfo*)info;
-(void)callITunes;
-(void)newTrackStarted:(NowPlayingInfo*)info;
-(void)iTunesNotRunning;
-(void)iTunesNotPlaying;
-(void)playingSameTrackAtPosition:(NSInteger)position ofDuration:(NSInteger)duration;
-(void)animateArtworkTransitionToImage:(NSImage*)image;


@property (retain, nonatomic) NowPlayingInfo* nowPlayingInfo;
@property (retain, nonatomic) NSImage* defaultImage;
@property (retain, nonatomic) iTunesApplication* player;

@end


@implementation NowPlayingSaverView

static inline BOOL empty(NSString* s)
{
    return (s == nil) || ([s length] == 0);
}

-(NSString*)formatTrackInfo:(NowPlayingInfo*)info
{
    NSMutableString* infoString = [[NSMutableString alloc] init];
    if (! empty(info.albumName))
    {
        if (! empty(info.albumArtistName))
            [infoString appendFormat:@"%@: ", info.albumArtistName];
        else if (! empty(info.artistName))
            [infoString appendFormat:@"%@: ", info.artistName];
            
        [infoString appendFormat:@"%@\n", info.albumName];
        
        if ((! [info.artistName isEqualToString:info.albumArtistName]) &&
            (! empty(info.albumArtistName)) &&
            (! empty(info.artistName)))
            [infoString appendFormat:@"%@: ", info.artistName];
    }
    else
    {
        if (! empty(info.artistName))
            [infoString appendFormat:@"%@: ", info.artistName];
    }

    [infoString appendFormat:@"%d - %@", info.trackNumber, ((info.trackName != nil) ? info.trackName : @"(no name)")];
    
    return infoString;
}


-(void)animateArtworkTransitionToImage:(NSImage*)image
{
    ArtworkAnimator* animator = [self.elementProvider element];
    [animator animateArtworkLayer:self.albumArtView.layer toArtwork:image];
}


-(void)newTrackStarted:(NowPlayingInfo*)info
{
    self.nowPlayingInfo = info;
    [self.albumInfoView setStringValue:[self formatTrackInfo:info]];
    NSImage* img = info.artwork != nil ? info.artwork : self.defaultImage;
//    [self.albumArtView setImage:img];
    [self animateArtworkTransitionToImage:img];
    [self.progressIndicator setDoubleValue:(info.position * 100.0) / info.duration];
    [self setNeedsDisplay:YES];
}


-(void)iTunesNotRunning
{
    self.nowPlayingInfo = nil;
    [self.albumInfoView setStringValue:@"iTunes is not running"];
    [self.albumArtView setImage:nil];
    [self.progressIndicator setDoubleValue:0];
}


-(void)iTunesNotPlaying
{
    self.nowPlayingInfo = nil;
    [self.albumInfoView setStringValue:@"iTunes is playing nothing"];
    [self.albumArtView setImage:nil];
    [self.progressIndicator setDoubleValue:0];
}


-(void)playingSameTrackAtPosition:(NSInteger)position ofDuration:(NSInteger)duration
{
    [self.progressIndicator setDoubleValue:(position * 100.0) / duration];
}


#define NOWS(s) [s stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]

-(void)callITunes
{
    NSInteger currentDbId = (self.nowPlayingInfo != nil) ? self.nowPlayingInfo.dbId : -1;
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        iTunesApplication* iTunes = self.player;
        NowPlayingInfo* info = [[NowPlayingInfo alloc] init];

        if (! [iTunes isRunning])
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self iTunesNotRunning];
            });
            return;
        }
                
        iTunesTrack* currentTrack = iTunes.currentTrack;
        
        if (! [currentTrack exists])
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self iTunesNotPlaying];
            });
            return;
        }
        
        NSInteger newId = currentTrack.databaseID;
        if (newId == currentDbId)
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self playingSameTrackAtPosition:iTunes.playerPosition ofDuration:currentTrack.duration]; 
            });
            return;
        }
        
        info.trackName = NOWS(currentTrack.name);
        info.trackNumber = currentTrack.trackNumber;
        info.artistName = NOWS(currentTrack.artist);
        info.albumName = NOWS(currentTrack.album);
        info.albumArtistName = NOWS(currentTrack.albumArtist);
        info.dbId = newId;
        info.duration = currentTrack.duration;
        info.position = iTunes.playerPosition;
        // TODO: which artwork should we pick and is this the best way
        NSArray* artworks = currentTrack.artworks;
        iTunesArtwork* artwork = ((artworks != nil) && [artworks count]) ? [artworks objectAtIndex:0] : nil;
        info.artwork = artwork != nil ? artwork.data : nil;

        dispatch_async(dispatch_get_main_queue(), ^{
            [self newTrackStarted:info]; 
        });
        
    });
}

- (id)initWithFrame:(NSRect)frame isPreview:(BOOL)isPreview
{
    if ((self = [super initWithFrame:frame isPreview:isPreview]))
    {
        [self setAnimationTimeInterval:0];
        self.nowPlayingInfo = nil;
        if (! [NSBundle loadNibNamed:@"iTunesDataView" owner:self])
        {
            NSLog(@"NowPlayingSaver nib loading failed");
        }
        [self addSubview:self.infoFrame];

        CALayer* layer = [CALayer layer];
        self.albumArtView.layer = layer;
        self.albumArtView.wantsLayer = YES;
        layer.bounds = self.albumArtView.bounds;
        layer.contentsGravity = kCAGravityResizeAspect;
        layer.anchorPoint = CGPointMake(0.5, 0.5);
        
        self.infoFrame.frame = self.bounds;
        NSString* itunesPath = [[NSWorkspace sharedWorkspace] fullPathForApplication:@"iTunes"];
        self.defaultImage = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithPath:itunesPath] pathForResource:@"iTunes" ofType:@"icns"]];
        self.elementProvider = [[RandomElementProvider alloc] 
                                initWithElements:
                                [NSArray arrayWithObjects:
                                 [[ArtworkFadeOutInAnimator alloc] init],
                                 [[ArtworkScaleDownUpAnimator alloc] init],
                                 [[ArtworkDrillAnimator alloc] init],
                                 [[ArtworkFlipAnimator alloc] initWithAxis:kArtworkFlipAxisX],
                                 [[ArtworkFlipAnimator alloc] initWithAxis:kArtworkFlipAxisY],
                                 [[ArtworkFromSideAnimator alloc] initWithDirection:kArtworkFromSideLeft],
                                 [[ArtworkFromSideAnimator alloc] initWithDirection:kArtworkFromSideRight],
                                 [[ArtworkCircleAnimator alloc] initWithDirection:kArtworkCircleClockwise],
                                 [[ArtworkCircleAnimator alloc] initWithDirection:kArtworkCircleCounterClockwise],
                                 [[ArtworkDoorwayAnimator alloc] init],
                                 nil]];
        srandomdev();
    }
    return self;
}

-(void)showFakeInfo:(NowPlayingInfo*)info
{
    [self newTrackStarted:info];
}

- (void)startAnimation
{
    [super startAnimation];
    self.player = [SBApplication applicationWithBundleIdentifier:@"com.apple.iTunes"];
}

- (void)stopAnimation
{
    [super stopAnimation];
}

- (void)drawRect:(NSRect)rect
{
    [super drawRect:rect];
}

-(NSTimeInterval)animationTimeInterval
{
    return 2;
}

- (void)animateOneFrame
{
    [self callITunes];
}

- (BOOL)hasConfigureSheet
{
    return NO;
}

- (NSWindow*)configureSheet
{
    return nil;
}

@synthesize albumArtView;
@synthesize albumInfoView;
@synthesize defaultImage;
@synthesize elementProvider;
@synthesize infoFrame;
@synthesize nowPlayingInfo;
@synthesize player;
@synthesize progressIndicator;

@end
